package sun.rmi.transport;

import java.rmi.RemoteException;
import java.io.DataOutputStream;
import sun.rmi.runtime.Log;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.IOException;
import java.util.HashMap;
import java.io.InputStream;
import java.rmi.server.UID;
import java.util.List;
import java.util.Map;
import sun.rmi.server.MarshalInputStream;

class ConnectionInputStream extends MarshalInputStream
{
    private boolean dgcAckNeeded;
    private Map<Endpoint, List<LiveRef>> incomingRefTable;
    private UID ackID;
    
    ConnectionInputStream(InputStream in) throws IOException {
        super(in);
        this.dgcAckNeeded = false;
        this.incomingRefTable = new HashMap(5);
    }
    
    void readID() throws IOException {
        this.ackID = UID.read(this);
    }
    
    void saveRef(LiveRef liveRef) {
        Endpoint endpoint = liveRef.getEndpoint();
        List<LiveRef> list = (List)this.incomingRefTable.get(endpoint);
        if (list == null) {
            list = new ArrayList();
            this.incomingRefTable.put(endpoint, list);
        }
        list.add(liveRef);
    }
    
    void discardRefs() {
        this.incomingRefTable.clear();
    }
    
    void registerRefs() throws IOException {
        if (!this.incomingRefTable.isEmpty()) {
            for (Map.Entry entry : this.incomingRefTable.entrySet()) {
                DGCClient.registerRefs((Endpoint)entry.getKey(), (List)entry.getValue());
            }
        }
    }
    
    void setAckNeeded() {
        this.dgcAckNeeded = true;
    }
    
    void done(Connection connection) {
        if (this.dgcAckNeeded) {
            Connection connection2 = null;
            Channel channel = null;
            boolean n = true;
            DGCImpl.dgcLog.log(Log.VERBOSE, "send ack");
            try {
                channel = connection.getChannel();
                connection2 = channel.newConnection();
                DataOutputStream out = new DataOutputStream(connection2.getOutputStream());
                out.writeByte(84);
                if (this.ackID == null) {
                    this.ackID = new UID();
                }
                this.ackID.write(out);
                connection2.releaseOutputStream();
                connection2.getInputStream().available();
                connection2.releaseInputStream();
            }
            catch (RemoteException ex) {
                n = false;
            }
            catch (IOException ex2) {
                n = false;
            }
            try {
                if (connection2 != null) {
                    channel.free(connection2, (boolean)n);
                }
            }
            catch (RemoteException ex3) {}
        }
    }
}
